--- sys/nfs/bootp_subr.c	2014-01-26 15:55:42.000000000 -0800
+++ /home/john/tmp/bootp_subr.c	2014-05-24 19:03:59.000000000 -0700
@@ -198,6 +198,7 @@
 
 /* DHCP specific tags */
 #define TAG_OVERLOAD	 52  /* Option Overload */
+#define TAG_PARAM_REQ    55  /* Tag denoting optional tags in DHCP client message, requesting data from DHCP server */
 #define TAG_MAXMSGSIZE   57  /* Maximum DHCP Message Size */
 
 #define TAG_END		255  /* End Option (i.e. no more options) */
@@ -573,7 +574,7 @@
 static int
 bootpc_call(struct bootpc_globalcontext *gctx, struct thread *td)
 {
-	struct sockaddr_in *sin, dst;
+	struct sockaddr_in *sin, dst, *from;
 	struct uio auio;
 	struct sockopt sopt;
 	struct iovec aio;
@@ -587,6 +588,7 @@
 	int retry;
 	const char *s;
 
+    from = (struct sockaddr_in *) 0;
 	tv.tv_sec = 1;
 	tv.tv_usec = 0;
 	bzero(&sopt, sizeof(sopt));
@@ -760,9 +762,14 @@
 		if (timo < MAX_RESEND_DELAY)
 			timo++;
 		else {
-			printf("DHCP/BOOTP timeout for server ");
-			print_sin_addr(&dst);
-			printf("\n");
+            if (from != (struct sockaddr_in *) 0) {
+                printf("DHCP/BOOTP timeout for server ");
+                print_sin_addr(from);
+                printf("\n");
+            }
+            else {
+                printf ("DHCP/BOOTP timeout\n");
+            }
 		}
 
 		/*
@@ -783,7 +790,11 @@
 			auio.uio_td = td;
 
 			rcvflg = 0;
-			error = soreceive(bootp_so, NULL, &auio,
+            if (from != (struct sockaddr_in *) 0) {
+                free (from, M_SONAME);
+                from = (struct sockaddr_in *) 0;
+            }
+			error = soreceive(bootp_so, (struct sockaddr **) &from, &auio,
 					  NULL, NULL, &rcvflg);
 			gctx->secs = time_second - gctx->starttime;
 			STAILQ_FOREACH(ifctx, &gctx->interfaces, next) {
@@ -850,7 +861,7 @@
 				       " on %s from ",
 				       s,
 				       ifctx->ireq.ifr_name);
-				print_in_addr(gctx->reply.siaddr);
+				print_sin_addr(from);
 				if (gctx->reply.giaddr.s_addr !=
 				    htonl(INADDR_ANY)) {
 					printf(" via ");
@@ -933,6 +944,10 @@
 	error = ETIMEDOUT;
 
 out:
+    if (from != (struct sockaddr_in *) 0) {
+        free (from, M_SONAME);
+        from = (struct sockaddr_in *) 0;
+    }
 	return (error);
 }
 
@@ -1277,6 +1292,15 @@
 		leasetime = htonl(300);
 		memcpy(vendp, &leasetime, 4);
 		vendp += 4;
+            
+        /*
+         * Request host and path of root disk (required for Windows Server DHCP)
+         */
+            
+        *vendp++ = TAG_PARAM_REQ;
+        *vendp++ = 2;
+        *vendp++ = TAG_HOSTNAME;
+        *vendp++ = TAG_ROOT;
 		break;
 	default:
 		break;
